function [Rand_Index, Normalized_Rand_Index, F_Measure]=ClusteringResultStatistics(LabelTruth, LabelDecision)
%---------------------------------------------------------------------------
%   Evaluate clustering result using Rand Index, Normalized Rand Index and F-Measure
%
%       Rand Index = # Correct Decisions / # All decisions
%
%       Noralized Rand Index = 0.5 * (# Correct same-cluster Decisions / # True same-cluster relationship)
%                            + 0.5 * (# Correct Different-cluster Decisions / # True Different-cluster relationship)
%
%       F_Measure = 2 .* (Precision .* Recall) ./ (Precision + Recall) where
%           Precision = nPairsCorrectlyPredictedInSameCluster ./ nTotalPairsPredictedInSameCluster;
%           Recall = nPairsCorrectlyPredictedInSameCluster ./ nTotalPairsInSameCluster;
%
%   Version 1.0
%   Peng Li @ 08 / 05 / 2007
%
%   function [Rand_Index, Normalized_Rand_Index, F_Measure] = ClusteringResultStatistics(LabelTruth, LabelDecision)
%
%   Note:
%       The labels must be continuous integers starting from 1
%       LabelTruth and LabelDecision must have the same length
%
%   Input:
%       LabelTruth      -   1 x na vector of true labels of the data
%       LabelDecision   -   1 x na vector of labels made by clustering algorithm
%   Output:
%       Rand_Index              -   Calculated Rand index (Favor assign to
%                                   different clusters)
%       Normalized_Rand_Index   -   Normalized Rand index which favor both
%       F_Measure               -   Calculated F-Measure 
%---------------------------------------------------------------------------
Rand_Index = [];
Normalized_Rand_Index = [];
F_Measure = [];

%   Process input variables
if nargin < 2 
    error('Error in the input labels!');
    return;
end
[nrow, ncol] = size(LabelTruth);
if nrow ~= 1
    if ncol ~= 1
        error('Error in the number of true labels!');
        return;
    end
    na = nrow;  % Number of samples
    LabelTruth = LabelTruth';   % Change to 1 x na vector
else
    na = ncol;  % Number of samples
end
[nrow, ncol] = size(LabelDecision);
if nrow ~= 1
    if ncol ~= 1
        error('Error in the number of decision labels!');
        return;
    end
    na1 = nrow;  % Number of decision labels
    LabelDecision = LabelDecision';   % Change to 1 x na vector
else
    na1 = ncol;  % Number of decision labels
end
if na1 ~= na
    error('The number of true labels and decisions must be the same.')'
end

% Calculate pair matrix of labels of ground truth
RowLabel = repmat(LabelTruth, na, 1);   % Each column is the same 
MTruth = RowLabel' - RowLabel;
MTruth(find(MTruth ~= 0)) = 1;          % 0 for same cluster, 1 for not

% Calculate pair matrix of labels of decision
RowLabel = repmat(LabelDecision, na, 1);% Each column is the same 
MDecision = RowLabel' - RowLabel;
MDecision(find(MDecision ~= 0)) = 1;    % 0 for same cluster, 1 for not

MDiffer = MDecision - MTruth;
MDiffer(find(MDiffer ~= 0)) = 1;

Rand_Index = 1 - sum(sum(MDiffer)) ./ (na .* (na - 1));

nSameClusterTruth = (length(find(MTruth == 0)) - na) ./ 2;
nDiffClusterTruth = length(find(MTruth == 1)) ./ 2;

nCorrectSameCluster = (length(find((MTruth == 0) & (MDecision == 0))) - na) ./ 2;
nCorrectDiffCluster = length(find((MTruth == 1) & (MDecision == 1))) ./ 2;

Normalized_Rand_Index = (nCorrectSameCluster ./ nSameClusterTruth + nCorrectDiffCluster ./ nDiffClusterTruth) ./ 2;

nPairsCorrectlyPredictedInSameCluster = (length(find((MTruth == 0) & (MDecision == 0))) - na) ./ 2;
nTotalPairsPredictedInSameCluster = (length(find(MDecision == 0)) - na) ./ 2;
nTotalPairsInSameCluster = (length(find(MTruth == 0)) - na) ./ 2;;

Precision = nPairsCorrectlyPredictedInSameCluster ./ nTotalPairsPredictedInSameCluster;
Recall = nPairsCorrectlyPredictedInSameCluster ./ nTotalPairsInSameCluster;

F_Measure = 2 .* (Precision .* Recall) ./ (Precision + Recall);
return